"""Utility functions."""

from __future__ import absolute_import, print_function

from io import BytesIO, StringIO
from nbformat.v3 import nbpy
import nbformat as nbf

from .reader import read_python


__all__ = ['py_string_to_nb_string', 'py_file_to_nb_string',
           'nb_string_to_notebook', 'write_notebook',
           'write_notebook_to_string', 'python_to_notebook']


def py_string_to_nb_string(str):
    """
    Read a string containing a regular Python script with special
    formatting, and perform preprocessing on it.  The result is a
    string that conforms to the IPython notebook version 3 Python
    script format.
    """

    return read_python(BytesIO(str.encode()))


def py_file_to_nb_string(filename):
    """
    Read a regular Python file with special formatting, and perform
    preprocessing on it.  The result is a string that conforms to the
    IPython notebook version 3 Python script format.
    """

    with open(filename, 'rb') as fin:
        ipy = read_python(fin)

    return ipy


def nb_string_to_notebook(str):
    """Convert a string representation of a notebook into a notebook
    object.
    """

    # Read using v3 of nbformat
    with StringIO(str) as fin:
        nb = nbpy.read(fin)

    return nb


def write_notebook(nb, filename):
    """Write a notebook object to a file."""

    # Write using the most recent version of nbformat
    with open(filename, 'w') as fout:
        nbf.write(nb, fout, version=nbf.current_nbformat)


def write_notebook_to_string(nb):
    """Write a notebook object to a string."""

    # Write using the most recent version of nbformat
    with StringIO('') as fout:
        nbf.write(nb, fout, version=nbf.current_nbformat)
        str = fout.getvalue()
    return str


def python_to_notebook(input_filename, output_filename):
    """Convert the given python source file into a properly formatted
    notebook.
    """

    nbstr = py_file_to_nb_string(input_filename)
    nb = nb_string_to_notebook(nbstr)
    write_notebook(nb, output_filename)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for py2jn module"""

import os
from tempfile import gettempdir
import pytest
import py2jn
from py2jn.__main__ import python_to_notebook


@pytest.fixture
def samplepy(fname='example.py'):
    return os.path.join('tests', fname)


@pytest.fixture
def sampleipynb(fname='example.ipynb'):
    return os.path.join('tests', fname)


def test_py2jn(samplepy, sampleipynb):
    tmpdir = gettempdir()
    outfile = os.path.join(tmpdir, 'py2jn_example.ipynb')
    python_to_notebook(input_filename=samplepy, output_filename=outfile)
    with open(os.path.join(tmpdir, 'py2jn_example.ipynb'), 'r') as outfileobj:
        outfile = outfileobj.read()
    with open(sampleipynb, 'r') as reffileobj:
        refile = reffileobj.read()
    assert outfile == refile
    os.remove(os.path.join(tmpdir, 'py2jn_example.ipynb'))


def test_str_to_str(samplepy, sampleipynb):
    with open(samplepy, 'r') as fin:
        pystr = fin.read()
    nbstr = py2jn.py_string_to_nb_string(pystr)
    nb = py2jn.nb_string_to_notebook(nbstr)
    outstr = py2jn.write_notebook_to_string(nb)
    with open(sampleipynb, 'r') as fin:
        refstr = fin.read()
    assert outstr == refstr



"""Read a Python file into string in IPython notebook form."""

from __future__ import absolute_import, division, print_function

import sys
import tokenize
from io import StringIO

if sys.version_info[0] == 2:
    # Python 2.x
    from collections import namedtuple
    TokenInfo = namedtuple('TokenInfo', 'type string start end line')

    def _generate_tokens(readline):
        return map(lambda x: TokenInfo(*x), tokenize.generate_tokens(readline))

else:
    # Python 3.x
    TokenInfo = tokenize.TokenInfo
    _generate_tokens = tokenize.tokenize



def read_python(fin):
    """
    Read a regular Python file with special formatting, and perform
    preprocessing on it.  The result is a string that conforms to the
    IPython notebook version 3 Python script format.
    """

    token_gen = _generate_tokens(fin.readline)
    cvt_docstr_gen = convert_toplevel_docstring(token_gen)
    nl_gen = fix_newlines(cvt_docstr_gen)
    out = list(nl_gen)
    formatted = tokenize.untokenize(out).decode('utf-8')

    return fix_empty_lines(formatted)


# ======================================================================
#                                   Helpers
# ======================================================================

def convert_toplevel_docstring(tokens):
    for token in tokens:
        # For each string
        if token.type == tokenize.STRING:
            text = token.string
            # Must be a docstring
            if text.startswith('"""') or text.startswith("'''"):
                startline, startcol = token.start
                # Starting column MUST be 0
                if startcol == 0:
                    endline, endcol = token.end
                    lines = ['# ' + line
                             for line in text.strip('"\' \n').split('\n')]
                    text = '\n'.join(lines)
                    fmt = '# <markdowncell>\n{0}\n# <codecell>'.format(text)
                    yield TokenInfo(type=tokenize.COMMENT,
                                    start=(startline, startcol),
                                    end=(endline, endcol),
                                    string=fmt,
                                    line='#')
                    # To next token
                    continue
        # Return untouched
        yield token


def fix_newlines(tokens):
    first = True
    curline = 1
    for token in tokens:
        if first:
            first = False
            curline = token.end[0] + 1
        else:
            # Fill NEWLINE token in between
            while curline < token.start[0]:
                yield TokenInfo(type=tokenize.NEWLINE,
                                string='\n',
                                start=(curline, 0),
                                end=(curline, 0),
                                line='\n', )
                curline += 1

            curline = token.end[0] + 1
        yield token


def fix_empty_lines(text):
    def gen():
        for line in text.splitlines():
            if not line.strip():
                # Empty line
                yield ''
            else:
                yield line

    return '\n'.join(gen())
